//!OpenSCAD

stockthickness = 18.5;
stockdepth = 88.5;
stockwidth = 33.5;
holderonetype = "round"; // [round, rectangular]
holderonelength = 6;
holderonewidth = 6;
holderonedepth = 18.5;
holderonecols = 2;
holderonerows = 2;
holderonespacing = 5.5;
holderoneoffset = 1;
holderoneborder = 0;
/* [Holder two] */
includeholdertwo = true;
holdertwotype = "rectangular"; // [round, rectangular]
holdertwolength = 5;
holdertwowidth = 10;
holdertwodepth = 5;
holdertwocols = 2;
holdertworows = 2;
holdertwospacing = 4.5;
holdertwooffset = 0;
holdertwoborder = 0;
/* [Holder three] */
includeholderthree = true;
holderthreetype = "rectangular"; // [round, rectangular]
holderthreelength = 11.5;
holderthreewidth = 4.5;
holderthreedepth = 12;
holderthreecols = 4;
holderthreerows = 1;
holderthreespacing = 3.1;
holderthreeoffset = 0;
holderthreeborder = 0;
/* [Holder four] */
includeholderfour = true;
holderfourtype = "round"; // [round, rectangular]
holderfourlength = 6;
holderfourwidth = 6;
holderfourdepth = 18.5;
holderfourcols = 4;
holderfourrows = 2;
holderfourspacing = 2;
holderfouroffset = 0;
holderfourborder = 0;

module makeslot(type, radius, length2, width, depth) {
  if (type == "round") {
    translate([radius, radius, 0]){
      cylinder(r1=radius, r2=radius, h=(depth * 2), center=false);
    }
  } else {
    cube([width, length2, (depth * 2)], center=false);
  }

}

module makeborder(width, llx, lly, urx, ury) {
  translate([0, 0, (width / 2)]){
    union(){
      hull(){
        translate([llx, lly, 0]){
          cylinder(r1=0, r2=width, h=width, center=true);
        }
        translate([llx, ury, 0]){
          cylinder(r1=0, r2=width, h=width, center=true);
        }
      }
      hull(){
        translate([llx, lly, 0]){
          cylinder(r1=0, r2=width, h=width, center=true);
        }
        translate([urx, lly, 0]){
          cylinder(r1=0, r2=width, h=width, center=true);
        }
      }
      hull(){
        translate([urx, lly, 0]){
          cylinder(r1=0, r2=width, h=width, center=true);
        }
        translate([urx, ury, 0]){
          cylinder(r1=0, r2=width, h=width, center=true);
        }
      }
      hull(){
        translate([llx, ury, 0]){
          cylinder(r1=0, r2=width, h=width, center=true);
        }
        translate([urx, ury, 0]){
          cylinder(r1=0, r2=width, h=width, center=true);
        }
      }
    }
  }
}

$fn=50;
holderoneradius = holderonewidth / 2;
holdertworadius = holdertwowidth / 2;
holderthreeradius = holderthreewidth / 2;
holderfourradius = holderfourwidth / 2;

difference() {
  cube([stockwidth, stockdepth, stockthickness], center=false);

  translate([holderonespacing, holderonespacing, (stockthickness - holderonedepth)]){
    union(){
      for (i = [0 : abs(1) : holderonecols - 1]) {
        translate([(i * (holderonespacing + holderonewidth)), 0, 0]){
          for (j = [0 : abs(1) : holderonerows - 1]) {
            if (j % 2 == 0) {
              translate([0, (j * (holderonespacing + holderonewidth)), 0]){
                makeslot(holderonetype, holderoneradius, holderonelength, holderonewidth, holderonedepth);
              }
            } else {
              translate([(holderonespacing * holderoneoffset), (j * (holderonespacing + holderonelength)), 0]){
                makeslot(holderonetype, holderoneradius, holderonelength, holderonewidth, holderonedepth);
              }
            }

          }

        }
      }

      if (holderoneborder > 0) {
        translate([0, 0, (stockthickness - holderoneborder * 0.45454545)]){
          if (holderoneoffset == 1) {
            makeborder(holderoneborder * 0.55, 0 - holderonespacing / 2, 0 - holderonespacing / 2, holderonecols * holderonewidth + (holderonecols + 0.5) * holderonespacing, holderonerows * holderonelength + (holderonerows - 0.5) * holderonespacing);
          } else {
            makeborder(holderoneborder * 0.55, 0 - holderonespacing / 2, 0 - holderonespacing / 2, holderonecols * holderonewidth + (holderonecols - 0.5) * holderonespacing, holderonerows * holderonelength + (holderonerows - 0.5) * holderonespacing);
          }

        }
      }

    }
  }
  if (includeholdertwo == true) {
    union(){
      translate([holdertwospacing, (holderonespacing * (holderonerows + 1) + holderonelength * holderonerows), (stockthickness - holdertwodepth)]){
        for (i = [0 : abs(1) : holdertwocols - 1]) {
          translate([(i * (holdertwospacing + holdertwowidth)), 0, 0]){
            for (j = [0 : abs(1) : holdertworows - 1]) {
              if (j % 2 == 0) {
                translate([0, (j * (holdertwospacing + holdertwowidth)), 0]){
                  makeslot(holdertwotype, holderthreeradius, holdertwolength, holdertwowidth, holdertwodepth);
                }
              } else {
                translate([(holdertwospacing * holdertwooffset), (j * (holdertwospacing + holdertwolength)), 0]){
                  makeslot(holdertwotype, holderthreeradius, holdertwolength, holdertwowidth, holdertwodepth);
                }
              }

            }

          }
        }

      }
      if (holdertwoborder > 0) {
        translate([holdertwospacing, (holderonerows * holderonelength + (holderonerows + 1) * holderonespacing), (stockthickness - holdertwoborder * 0.45454545)]){
          if (holdertwooffset == 1) {
            makeborder(holdertwoborder * 0.55, 0 - holdertwospacing / 2, 0 - holdertwospacing / 2, holdertwocols * holdertwowidth + (holdertwocols + 0.5) * holdertwospacing, holdertworows * holdertwolength + (holdertworows - 0.5) * holdertwospacing);
          } else {
            makeborder(holdertwoborder * 0.55, 0 - holdertwospacing / 2, 0 - holdertwospacing / 2, holdertwocols * holdertwowidth + (holdertwocols - 0.5) * holdertwospacing, holdertworows * holdertwolength + (holdertworows - 0.5) * holdertwospacing);
          }

        }
      }

    }
  }

  if (includeholderthree == true) {
    translate([holderthreespacing, ((holdertwospacing * (holdertworows + 1) + holdertwolength * holdertworows) + (holderonespacing * (holderonerows + 1) + holderonelength * holderonerows)), (stockthickness - holderthreedepth)]){
      union(){
        for (i = [0 : abs(1) : holderthreecols - 1]) {
          translate([(i * (holderthreespacing + holderthreewidth)), 0, 0]){
            for (j = [0 : abs(1) : holderthreerows - 1]) {
              if (j % 2 == 0) {
                translate([0, (j * (holderthreespacing + holderthreewidth)), 0]){
                  makeslot(holderthreetype, holderthreeradius, holderthreelength, holderthreewidth, holderthreedepth);
                }
              } else {
                translate([(holderthreespacing * holderthreeoffset), (j * (holderthreespacing + holderthreelength)), 0]){
                  makeslot(holderthreetype, holderthreeradius, holderthreelength, holderthreewidth, holderthreedepth);
                }
              }

            }

          }
        }

        if (holderthreeborder > 0) {
          translate([0, 0, (holderthreedepth - holderthreeborder * 0.45454545)]){
            if (holderthreeoffset == 1) {
              makeborder(holderthreeborder * 0.55, 0 - holderthreespacing / 2, 0 - holderthreespacing / 2, holderthreecols * holderthreewidth + (holderthreecols + 0.5) * holderthreespacing, holderthreerows * holderthreelength + (holderthreerows - 0.5) * holderthreespacing);
            } else {
              makeborder(holderthreeborder * 0.55, 0 - holderthreespacing / 2, 0 - holderthreespacing / 2, holderthreecols * holderthreewidth + (holderthreecols - 0.5) * holderthreespacing, holderthreerows * holderthreelength + (holderthreerows - 0.5) * holderthreespacing);
            }

          }
        }

      }
    }
  }

  if (includeholderfour == true) {
    translate([holderfourspacing, ((holderthreespacing * (holderthreerows + 1) + holderthreelength * holderthreerows) + ((holdertwospacing * (holdertworows + 1) + holdertwolength * holdertworows) + (holderonespacing * (holderonerows + 1) + holderonelength * holderonerows))), (stockthickness - holderfourdepth)]){
      union(){
        for (i = [0 : abs(1) : holderfourcols - 1]) {
          translate([(i * (holderfourspacing + holderfourwidth)), 0, 0]){
            for (j = [0 : abs(1) : holderfourrows - 1]) {
              if (j % 2 == 0) {
                translate([0, (j * (holderfourspacing + holderfourwidth)), 0]){
                  makeslot(holderfourtype, holderfourradius, holderfourlength, holderfourwidth, holderfourdepth);
                }
              } else {
                translate([(holderfourspacing * holderfouroffset), (j * (holderfourspacing + holderfourlength)), 0]){
                  makeslot(holderfourtype, holderfourradius, holderfourlength, holderfourwidth, holderfourdepth);
                }
              }

            }

          }
        }

        if (holderfourborder > 0) {
          translate([0, 0, (holderfourdepth - holderfourborder * 0.45454545)]){
            if (holderfouroffset == 1) {
              makeborder(holderfourborder * 0.55, 0 - holderfourspacing / 2, 0 - holderfourspacing / 2, holderfourcols * holderfourwidth + (holderfourcols + 0.5) * holderfourspacing, holderfourrows * holderfourlength + (holderfourrows - 0.5) * holderfourspacing);
            } else {
              makeborder(holderfourborder * 0.55, 0 - holderfourspacing / 2, 0 - holderfourspacing / 2, holderfourcols * holderfourwidth + (holderfourcols - 0.5) * holderfourspacing, holderfourrows * holderfourlength + (holderfourrows - 0.5) * holderfourspacing);
            }

          }
        }

      }
    }
  }

}